@@ -7,6 +7,9 @@
 
 namespace rumur {
 
+class BaseTraversal;
+class ConstBaseTraversal;
+
 struct Node {
 
   location loc;
@@ -23,6 +26,9 @@ struct Node {
    */
   virtual void validate() const { }
 
+  virtual void visit(BaseTraversal &visitor) = 0;
+  virtual void visit(ConstBaseTraversal &visitor) const = 0;
+
 };
 
 }
