@@ -50,13 +50,6 @@ class Symtab {
     }
     throw Error("unknown symbol: " + name, loc);
   }
-
-  // Whether we are in the top-level scope.
-  __attribute__((deprecated("this will be removed in a future release")))
-  bool is_global_scope() const {
-    return scope.size() == 1;
-  }
-
 };
 
 }
