@@ -8,6 +8,7 @@
 #include <rumur/Expr.h>
 #include <rumur/Number.h>
 #include <rumur/Ptr.h>
+#include <rumur/traverse.h>
 #include <rumur/TypeExpr.h>
 #include <stdexcept>
 #include <string>
@@ -28,6 +29,14 @@ Number *Number::clone() const {
   return new Number(*this);
 }
 
+void Number::visit(BaseTraversal &visitor) {
+  visitor.visit_number(*this);
+}
+
+void Number::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_number(*this);
+}
+
 bool Number::constant() const {
   return true;
 }
