@@ -12,7 +12,6 @@
 #include <rumur/Stmt.h>
 #include <rumur/traverse.h>
 #include <string>
-#include "utils.h"
 #include <vector>
 
 namespace rumur {
@@ -66,6 +65,14 @@ AliasRule *AliasRule::clone() const {
   return new AliasRule(*this);
 }
 
+void AliasRule::visit(BaseTraversal &visitor) {
+  visitor.visit_aliasrule(*this);
+}
+
+void AliasRule::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_aliasrule(*this);
+}
+
 std::vector<Ptr<Rule>> AliasRule::flatten() const {
   std::vector<Ptr<Rule>> rs;
   for (const Ptr<Rule> &r : rules) {
@@ -90,6 +97,14 @@ void SimpleRule::validate() const {
   ReturnChecker::check(*this);
 }
 
+void SimpleRule::visit(BaseTraversal &visitor) {
+  visitor.visit_simplerule(*this);
+}
+
+void SimpleRule::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_simplerule(*this);
+}
+
 StartState::StartState(const std::string &name_,
   const std::vector<Ptr<Decl>> &decls_,
   const std::vector<Ptr<Stmt>> &body_, const location &loc_):
@@ -103,6 +118,14 @@ void StartState::validate() const {
   ReturnChecker::check(*this);
 }
 
+void StartState::visit(BaseTraversal &visitor) {
+  visitor.visit_startstate(*this);
+}
+
+void StartState::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_startstate(*this);
+}
+
 PropertyRule::PropertyRule(const std::string &name_, const Property &property_,
   const location &loc_):
   Rule(name_, loc_), property(property_) { }
@@ -111,6 +134,14 @@ PropertyRule *PropertyRule::clone() const {
   return new PropertyRule(*this);
 }
 
+void PropertyRule::visit(BaseTraversal &visitor) {
+  visitor.visit_propertyrule(*this);
+}
+
+void PropertyRule::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_propertyrule(*this);
+}
+
 Ruleset::Ruleset(const std::vector<Quantifier> &quantifiers_,
   const std::vector<Ptr<Rule>> &rules_, const location &loc_):
   Rule("", loc_), rules(rules_) {
@@ -129,6 +160,14 @@ void Ruleset::validate() const {
   }
 }
 
+void Ruleset::visit(BaseTraversal &visitor) {
+  visitor.visit_ruleset(*this);
+}
+
+void Ruleset::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_ruleset(*this);
+}
+
 std::vector<Ptr<Rule>> Ruleset::flatten() const {
   std::vector<Ptr<Rule>> rs;
   for (const Ptr<Rule> &r : rules) {
