@@ -9,10 +9,10 @@
 #include <rumur/Property.h>
 #include <rumur/Ptr.h>
 #include <rumur/Stmt.h>
+#include <rumur/traverse.h>
 #include <rumur/TypeExpr.h>
 #include <string>
 #include <utility>
-#include "utils.h"
 #include <vector>
 
 namespace rumur {
@@ -27,6 +27,14 @@ AliasStmt *AliasStmt::clone() const {
   return new AliasStmt(*this);
 }
 
+void AliasStmt::visit(BaseTraversal &visitor) {
+  visitor.visit_aliasstmt(*this);
+}
+
+void AliasStmt::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_aliasstmt(*this);
+}
+
 PropertyStmt::PropertyStmt(const Property &property_,
   const std::string &message_, const location &loc_):
   Stmt(loc_), property(property_), message(message_) { }
@@ -41,6 +49,14 @@ void PropertyStmt::validate() const {
       "level property", loc);
 }
 
+void PropertyStmt::visit(BaseTraversal &visitor) {
+  visitor.visit_propertystmt(*this);
+}
+
+void PropertyStmt::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_propertystmt(*this);
+}
+
 Assignment::Assignment(const Ptr<Expr> &lhs_, const Ptr<Expr> &rhs_,
   const location &loc_):
   Stmt(loc_), lhs(lhs_), rhs(rhs_) { }
@@ -61,6 +77,14 @@ void Assignment::validate() const {
     throw Error("invalid assignment from incompatible type", loc);
 }
 
+void Assignment::visit(BaseTraversal &visitor) {
+  visitor.visit_assignment(*this);
+}
+
+void Assignment::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_assignment(*this);
+}
+
 Clear::Clear(const Ptr<Expr> &rhs_, const location &loc_):
   Stmt(loc_), rhs(rhs_) { }
 
@@ -76,6 +100,14 @@ void Clear::validate() const {
     throw Error("invalid clear of read-only expression", loc);
 }
 
+void Clear::visit(BaseTraversal &visitor) {
+  visitor.visit_clear(*this);
+}
+
+void Clear::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_clear(*this);
+}
+
 ErrorStmt::ErrorStmt(const std::string &message_, const location &loc_):
   Stmt(loc_), message(message_) { }
 
@@ -83,6 +115,14 @@ ErrorStmt *ErrorStmt::clone() const {
   return new ErrorStmt(*this);
 }
 
+void ErrorStmt::visit(BaseTraversal &visitor) {
+  visitor.visit_errorstmt(*this);
+}
+
+void ErrorStmt::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_errorstmt(*this);
+}
+
 For::For(const Quantifier &quantifier_,
   const std::vector<Ptr<Stmt>> &body_, const location &loc_):
   Stmt(loc_), quantifier(quantifier_), body(body_) { }
@@ -91,6 +131,14 @@ For *For::clone() const {
   return new For(*this);
 }
 
+void For::visit(BaseTraversal &visitor) {
+  visitor.visit_for(*this);
+}
+
+void For::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_for(*this);
+}
+
 IfClause::IfClause(const Ptr<Expr> &condition_,
   const std::vector<Ptr<Stmt>> &body_, const location &loc_):
   Node(loc_), condition(condition_), body(body_) { }
@@ -111,6 +159,14 @@ void IfClause::validate() const {
     throw Error("condition of if clause is not a boolean expression", loc);
 }
 
+void IfClause::visit(BaseTraversal &visitor) {
+  visitor.visit_ifclause(*this);
+}
+
+void IfClause::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_ifclause(*this);
+}
+
 If::If(const std::vector<IfClause> &clauses_, const location &loc_):
   Stmt(loc_), clauses(clauses_) { }
 
@@ -118,6 +174,14 @@ If *If::clone() const {
   return new If(*this);
 }
 
+void If::visit(BaseTraversal &visitor) {
+  visitor.visit_if(*this);
+}
+
+void If::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_if(*this);
+}
+
 ProcedureCall::ProcedureCall(const std::string &name,
   const std::vector<Ptr<Expr>> &arguments, const location &loc_):
   Stmt(loc_), call(name, arguments, loc_) {
@@ -129,6 +193,14 @@ ProcedureCall *ProcedureCall::clone() const {
   return new ProcedureCall(*this);
 }
 
+void ProcedureCall::visit(BaseTraversal &visitor) {
+  visitor.visit_procedurecall(*this);
+}
+
+void ProcedureCall::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_procedurecall(*this);
+}
+
 Put::Put(const std::string &value_, const location &loc_):
   Stmt(loc_), value(value_) { }
 
@@ -144,6 +216,14 @@ void Put::validate() const {
     throw Error("printing a complex non-lvalue is not supported", loc);
 }
 
+void Put::visit(BaseTraversal &visitor) {
+  visitor.visit_put(*this);
+}
+
+void Put::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_put(*this);
+}
+
 Return::Return(const Ptr<Expr> &expr_, const location &loc_):
   Stmt(loc_), expr(expr_) { }
 
@@ -151,6 +231,14 @@ Return *Return::clone() const {
   return new Return(*this);
 }
 
+void Return::visit(BaseTraversal &visitor) {
+  visitor.visit_return(*this);
+}
+
+void Return::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_return(*this);
+}
+
 SwitchCase::SwitchCase(const std::vector<Ptr<Expr>> &matches_,
     const std::vector<Ptr<Stmt>> &body_, const location &loc_):
   Node(loc_), matches(matches_), body(body_) { }
@@ -159,6 +247,14 @@ SwitchCase *SwitchCase::clone() const {
   return new SwitchCase(*this);
 }
 
+void SwitchCase::visit(BaseTraversal &visitor) {
+  visitor.visit_switchcase(*this);
+}
+
+void SwitchCase::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_switchcase(*this);
+}
+
 Switch::Switch(const Ptr<Expr> &expr_, const std::vector<SwitchCase> &cases_,
     const location &loc_):
   Stmt(loc_), expr(expr_), cases(cases_) { }
@@ -182,6 +278,14 @@ void Switch::validate() const {
   }
 }
 
+void Switch::visit(BaseTraversal &visitor) {
+  visitor.visit_switch(*this);
+}
+
+void Switch::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_switch(*this);
+}
+
 Undefine::Undefine(const Ptr<Expr> &rhs_, const location &loc_):
   Stmt(loc_), rhs(rhs_) { }
 
@@ -197,6 +301,14 @@ void Undefine::validate() const {
     throw Error("invalid undefine of read-only expression", loc);
 }
 
+void Undefine::visit(BaseTraversal &visitor) {
+  visitor.visit_undefine(*this);
+}
+
+void Undefine::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_undefine(*this);
+}
+
 While::While(const Ptr<Expr> &condition_, const std::vector<Ptr<Stmt>> &body_,
     const location &loc_):
   Stmt(loc_), condition(condition_), body(body_) { }
@@ -211,4 +323,12 @@ void While::validate() const {
       condition->loc);
 }
 
+void While::visit(BaseTraversal &visitor) {
+  visitor.visit_while(*this);
+}
+
+void While::visit(ConstBaseTraversal &visitor) const {
+  visitor.visit_while(*this);
+}
+
 }
