@@ -24,20 +24,7 @@ Ptr<Model> parse(std::istream &input) {
   if (err != 0)
     throw Error("parsing failed", location());
 
-  // mark the global declarations
-  for (Ptr<Decl> &d : m->decls) {
-    if (auto v = dynamic_cast<VarDecl*>(&*d)) {
-      v->state_variable = true;
-    }
-  }
-
   return m;
 }
 
-Ptr<Model> parse(std::istream *input) {
-  assert(input != nullptr);
-
-  return parse(*input);
-}
-
 }
