@@ -1,359 +1,20 @@
-#include <cassert>
 #include <cstddef>
 #include <iostream>
 #include <memory>
 #include <rumur/Expr.h>
 #include <rumur/Model.h>
+#include <rumur/Node.h>
 #include <rumur/Number.h>
+#include <rumur/Ptr.h>
 #include <rumur/Rule.h>
 #include <rumur/Stmt.h>
 #include <rumur/traverse.h>
 #include <rumur/TypeExpr.h>
-#include <typeinfo>
 
 namespace rumur {
 
 void BaseTraversal::dispatch(Node &n) {
-
-  if (auto i = dynamic_cast<Add*>(&n)) {
-    visit_add(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<AliasDecl*>(&n)) {
-    visit_aliasdecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<AliasRule*>(&n)) {
-    visit_aliasrule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<AliasStmt*>(&n)) {
-    visit_aliasstmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<AmbiguousAmp*>(&n)) {
-    visit_ambiguousamp(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<AmbiguousPipe*>(&n)) {
-    visit_ambiguouspipe(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<And*>(&n)) {
-    visit_and(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Array*>(&n)) {
-    visit_array(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Assignment*>(&n)) {
-    visit_assignment(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Band*>(&n)) {
-    visit_band(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Bnot*>(&n)) {
-    visit_bnot(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Bor*>(&n)) {
-    visit_bor(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<ConstDecl*>(&n)) {
-    visit_constdecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Clear*>(&n)) {
-    visit_clear(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Div*>(&n)) {
-    visit_div(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Element*>(&n)) {
-    visit_element(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Function*>(&n)) {
-    visit_function(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<FunctionCall*>(&n)) {
-    visit_functioncall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Enum*>(&n)) {
-    visit_enum(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Eq*>(&n)) {
-    visit_eq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<ErrorStmt*>(&n)) {
-    visit_errorstmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Exists*>(&n)) {
-    visit_exists(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<ExprID*>(&n)) {
-    visit_exprid(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Field*>(&n)) {
-    visit_field(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<For*>(&n)) {
-    visit_for(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Forall*>(&n)) {
-    visit_forall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Geq*>(&n)) {
-    visit_geq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Gt*>(&n)) {
-    visit_gt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<If*>(&n)) {
-    visit_if(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<IfClause*>(&n)) {
-    visit_ifclause(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Implication*>(&n)) {
-    visit_implication(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<IsUndefined*>(&n)) {
-    visit_isundefined(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Leq*>(&n)) {
-    visit_leq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Lsh*>(&n)) {
-    visit_lsh(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Lt*>(&n)) {
-    visit_lt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Model*>(&n)) {
-    visit_model(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Mod*>(&n)) {
-    visit_mod(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Mul*>(&n)) {
-    visit_mul(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Negative*>(&n)) {
-    visit_negative(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Neq*>(&n)) {
-    visit_neq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Not*>(&n)) {
-    visit_not(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Number*>(&n)) {
-    visit_number(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Or*>(&n)) {
-    visit_or(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<ProcedureCall*>(&n)) {
-    visit_procedurecall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Property*>(&n)) {
-    visit_property(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<PropertyRule*>(&n)) {
-    visit_propertyrule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<PropertyStmt*>(&n)) {
-    visit_propertystmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Put*>(&n)) {
-    visit_put(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Quantifier*>(&n)) {
-    visit_quantifier(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Range*>(&n)) {
-    visit_range(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Record*>(&n)) {
-    visit_record(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Return*>(&n)) {
-    visit_return(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Rsh*>(&n)) {
-    visit_rsh(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Ruleset*>(&n)) {
-    visit_ruleset(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Scalarset*>(&n)) {
-    visit_scalarset(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<SimpleRule*>(&n)) {
-    visit_simplerule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<StartState*>(&n)) {
-    visit_startstate(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Sub*>(&n)) {
-    visit_sub(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Switch*>(&n)) {
-    visit_switch(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<SwitchCase*>(&n)) {
-    visit_switchcase(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Ternary*>(&n)) {
-    visit_ternary(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<TypeDecl*>(&n)) {
-    visit_typedecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<TypeExprID*>(&n)) {
-    visit_typeexprid(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Undefine*>(&n)) {
-    visit_undefine(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<VarDecl*>(&n)) {
-    visit_vardecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<While*>(&n)) {
-    visit_while(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<Xor*>(&n)) {
-    visit_xor(*i);
-    return;
-  }
-
-#ifndef NDEBUG
-  std::cerr << "missed case in BaseTraversal::dispatch: " << typeid(n).name() << "\n";
-#endif
-  assert(!"missed case in BaseTraversal::dispatch");
+  n.visit(*this);
 }
 
 void BaseTraversal::visit_ambiguousamp(AmbiguousAmp &n) {
@@ -527,12 +188,8 @@ void Traversal::visit_mod(Mod &n) {
 }
 
 void Traversal::visit_model(Model &n) {
-  for (auto &d : n.decls)
-    dispatch(*d);
-  for (auto &f : n.functions)
-    dispatch(*f);
-  for (auto &r : n.rules)
-    dispatch(*r);
+  for (Ptr<Node> &c : n.children)
+    dispatch(*c);
 }
 
 void Traversal::visit_mul(Mul &n) {
@@ -696,346 +353,7 @@ void Traversal::visit_xor(Xor &n) {
 Traversal::~Traversal() { }
 
 void ConstBaseTraversal::dispatch(const Node &n) {
-
-  if (auto i = dynamic_cast<const Add*>(&n)) {
-    visit_add(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const AliasDecl*>(&n)) {
-    visit_aliasdecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const AliasRule*>(&n)) {
-    visit_aliasrule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const AliasStmt*>(&n)) {
-    visit_aliasstmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const AmbiguousAmp*>(&n)) {
-    visit_ambiguousamp(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const AmbiguousPipe*>(&n)) {
-    visit_ambiguouspipe(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const And*>(&n)) {
-    visit_and(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Array*>(&n)) {
-    visit_array(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Assignment*>(&n)) {
-    visit_assignment(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Band*>(&n)) {
-    visit_band(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Bnot*>(&n)) {
-    visit_bnot(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Bor*>(&n)) {
-    visit_bor(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Clear*>(&n)) {
-    visit_clear(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const ConstDecl*>(&n)) {
-    visit_constdecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Div*>(&n)) {
-    visit_div(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Element*>(&n)) {
-    visit_element(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Enum*>(&n)) {
-    visit_enum(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Eq*>(&n)) {
-    visit_eq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const ErrorStmt*>(&n)) {
-    visit_errorstmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Exists*>(&n)) {
-    visit_exists(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const ExprID*>(&n)) {
-    visit_exprid(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Field*>(&n)) {
-    visit_field(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const For*>(&n)) {
-    visit_for(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Forall*>(&n)) {
-    visit_forall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Function*>(&n)) {
-    visit_function(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const FunctionCall*>(&n)) {
-    visit_functioncall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Geq*>(&n)) {
-    visit_geq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Gt*>(&n)) {
-    visit_gt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const If*>(&n)) {
-    visit_if(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const IfClause*>(&n)) {
-    visit_ifclause(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Implication*>(&n)) {
-    visit_implication(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const IsUndefined*>(&n)) {
-    visit_isundefined(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Leq*>(&n)) {
-    visit_leq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Lsh*>(&n)) {
-    visit_lsh(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Lt*>(&n)) {
-    visit_lt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Model*>(&n)) {
-    visit_model(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Mod*>(&n)) {
-    visit_mod(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Mul*>(&n)) {
-    visit_mul(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Negative*>(&n)) {
-    visit_negative(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Neq*>(&n)) {
-    visit_neq(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Not*>(&n)) {
-    visit_not(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Number*>(&n)) {
-    visit_number(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Or*>(&n)) {
-    visit_or(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const ProcedureCall*>(&n)) {
-    visit_procedurecall(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Property*>(&n)) {
-    visit_property(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const PropertyRule*>(&n)) {
-    visit_propertyrule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const PropertyStmt*>(&n)) {
-    visit_propertystmt(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Put*>(&n)) {
-    visit_put(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Quantifier*>(&n)) {
-    visit_quantifier(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Range*>(&n)) {
-    visit_range(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Record*>(&n)) {
-    visit_record(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Return*>(&n)) {
-    visit_return(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Rsh*>(&n)) {
-    visit_rsh(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Ruleset*>(&n)) {
-    visit_ruleset(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Scalarset*>(&n)) {
-    visit_scalarset(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const SimpleRule*>(&n)) {
-    visit_simplerule(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const StartState*>(&n)) {
-    visit_startstate(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Sub*>(&n)) {
-    visit_sub(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Switch*>(&n)) {
-    visit_switch(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const SwitchCase*>(&n)) {
-    visit_switchcase(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Ternary*>(&n)) {
-    visit_ternary(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const TypeDecl*>(&n)) {
-    visit_typedecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const TypeExprID*>(&n)) {
-    visit_typeexprid(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Undefine*>(&n)) {
-    visit_undefine(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const VarDecl*>(&n)) {
-    visit_vardecl(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const While*>(&n)) {
-    visit_while(*i);
-    return;
-  }
-
-  if (auto i = dynamic_cast<const Xor*>(&n)) {
-    visit_xor(*i);
-    return;
-  }
-
-#ifndef NDEBUG
-  std::cerr << "missed case in ConstBaseTraversal::dispatch: " << typeid(n).name() << "\n";
-#endif
-  assert(!"missed case in ConstBaseTraversal::dispatch");
+  n.visit(*this);
 }
 
 void ConstBaseTraversal::visit_ambiguousamp(const AmbiguousAmp &n) {
@@ -1209,12 +527,8 @@ void ConstTraversal::visit_mod(const Mod &n) {
 }
 
 void ConstTraversal::visit_model(const Model &n) {
-  for (auto &d : n.decls)
-    dispatch(*d);
-  for (auto &f : n.functions)
-    dispatch(*f);
-  for (auto &r : n.rules)
-    dispatch(*r);
+  for (const Ptr<Node> &c : n.children)
+    dispatch(*c);
 }
 
 void ConstTraversal::visit_mul(const Mul &n) {
@@ -1447,12 +761,8 @@ void ConstExprTraversal::visit_ifclause(const IfClause &n) {
 }
 
 void ConstExprTraversal::visit_model(const Model &n) {
-  for (auto &d : n.decls)
-    dispatch(*d);
-  for (auto &f : n.functions)
-    dispatch(*f);
-  for (auto &r : n.rules)
-    dispatch(*r);
+  for (const Ptr<Node> &c : n.children)
+    dispatch(*c);
 }
 
 void ConstExprTraversal::visit_procedurecall(const ProcedureCall &n) {
@@ -1701,12 +1011,8 @@ void ConstStmtTraversal::visit_mod(const Mod &n) {
 }
 
 void ConstStmtTraversal::visit_model(const Model &n) {
-  for (auto &d : n.decls)
-    dispatch(*d);
-  for (auto &f : n.functions)
-    dispatch(*f);
-  for (auto &r : n.rules)
-    dispatch(*r);
+  for (const Ptr<Node> &c : n.children)
+    dispatch(*c);
 }
 
 void ConstStmtTraversal::visit_mul(const Mul &n) {
@@ -1987,12 +1293,8 @@ void ConstTypeTraversal::visit_mod(const Mod &n) {
 }
 
 void ConstTypeTraversal::visit_model(const Model &n) {
-  for (auto &d : n.decls)
-    dispatch(*d);
-  for (auto &f : n.functions)
-    dispatch(*f);
-  for (auto &r : n.rules)
-    dispatch(*r);
+  for (const Ptr<Node> &c : n.children)
+    dispatch(*c);
 }
 
 void ConstTypeTraversal::visit_mul(const Mul &n) {
