@@ -1,5 +1,6 @@
 #include "assume-statements-count.h"
 #include <cassert>
+#include "../../common/isa.h"
 #include <cstddef>
 #include <fstream>
 #include <iostream>
@@ -12,7 +13,6 @@
 #include <string>
 #include "symmetry-reduction.h"
 #include <utility>
-#include "utils.h"
 #include "ValueType.h"
 
 using namespace rumur;
@@ -105,23 +105,25 @@ static mpz_class rule_taken_max(const Model &model,
   mpz_class total = 0;
 
   // flatten all rules
-  for (const Ptr<Rule> &r : model.rules) {
-    const std::vector<Ptr<Rule>> flattened = r->flatten();
-    for (const Ptr<Rule> &f : flattened) {
+  for (const Ptr<Node> &c : model.children) {
+    if (auto r = dynamic_cast<const Rule*>(c.get())) {
+      const std::vector<Ptr<Rule>> flattened = r->flatten();
+      for (const Ptr<Rule> &f : flattened) {
 
-      // skip anything that does not match our criteria
-      if (!predicate(*f))
-        continue;
+        // skip anything that does not match our criteria
+        if (!predicate(*f))
+          continue;
 
-      // determine how many rules this generates when quantifiers are expanded
-      mpz_class rules = 1;
-      for (const Quantifier &q : f->quantifiers) {
-        assert(q.constant() && "non-constant quantifier used in rule "
-          "(unvalidated AST?)");
-        rules *= q.count();
+        // determine how many rules this generates when quantifiers are expanded
+        mpz_class rules = 1;
+        for (const Quantifier &q : f->quantifiers) {
+          assert(q.constant() && "non-constant quantifier used in rule "
+            "(unvalidated AST?)");
+          rules *= q.count();
+        }
+
+        total += rules;
       }
-
-      total += rules;
     }
   }
 
@@ -241,7 +243,8 @@ int output_checker(const std::string &path, const Model &model,
     << "/* whether scalarset schedules should be computed and used during printing */\n"
     << "#define USE_SCALARSET_SCHEDULES (" << (options.scalarset_schedules ? "1" : "0")
       << " && SYMMETRY_REDUCTION != SYMMETRY_REDUCTION_OFF && \\\n"
-    << "  (COUNTEREXAMPLE_TRACE != CEX_OFF || PRINTS_SCALARSETS))\n";
+    << "  (COUNTEREXAMPLE_TRACE != CEX_OFF || PRINTS_SCALARSETS))\n"
+    << "#define POINTER_BITS " << options.pointer_bits << "\n";
 
   generate_cover_array(out, model);
 
