@@ -5,7 +5,11 @@
 -- this feature. E.g. a first attempt at implementing iteration for array
 -- printing made the mistake of inaccurate indexing resulting in "z[6]:true".
 
+type
+  e: enum { A, B };
+
 var
+  w: e;
   x: boolean;
   y: record
     a: boolean;
@@ -13,6 +17,9 @@ var
   end;
   z: array [-4.. 7] of boolean;
 
+  -- a more complex type
+  c: array[boolean] of record r: 0..1; s: array[3..4] of 0..3; end;
+
 startstate begin
 
   put "printing an uninitialised boolean...\n";
@@ -52,6 +59,29 @@ startstate begin
   put z;
   put "\n";
 
+  put "printing an uninitialised enum...\n";
+  put w;
+  put "\n";
+
+  w := B;
+  put "printing an initialised enum...\n";
+  put w;
+  put "\n";
+
+  put "printing an uninitialised complex type...\n";
+  put c;
+  put "\n";
+
+  c[false].r := 0;
+  c[false].s[3] := 1;
+  c[false].s[4] := 2;
+  c[true].r := 1;
+  c[true].s[3] := 2;
+  c[true].s[4] := 3;
+  put "printing an initialised complex type...\n";
+  put c;
+  put "\n";
+
 end;
 
 rule begin
