@@ -1,8 +1,8 @@
--- rumur_flags: SMT_ARGS
--- skip_reason: 'no SMT solver available' if len(SMT_ARGS) == 0 else None
+-- rumur_flags: self.config['SMT_ARGS']
+-- skip_reason: 'no SMT solver available' if self.config['SMT_ARGS'] is None else None
 
 /* test whether the SMT bridge can simplify expressions involving arrays with
- * boolean indicies
+ * boolean indices
  */
 
 type
