@@ -1,5 +1,5 @@
 -- skip_reason: 'N/A in XML mode' if self.xml else 'N/A in non-debug mode' if hasattr(self, 'debug') and not self.debug else None
--- checker_output: re.compile(r'\bfield y is located at state offset 0 bits$(.|\n)*\bfield x is located at state offset 2 bits$', re.MULTILINE)
+-- checker_output: re.compile(r'\bfield x is located at state offset 2 bits$(.|\n)*\bfield y is located at state offset 0 bits$', re.MULTILINE)
 
 -- This tests for a regression of a problem first observed on commit
 -- 4944427734628cf913e8d5eeb54d897033f9eb59. When state variables should have
